﻿using System;
using System.Drawing;
using System.Windows.Forms;
using Ozeki.Media;

namespace MotionDetectorSample
{
    public partial class MotionDetectorForm : Form
    {
        DrawingImageProvider _imageProvider;
        VideoViewerWF _videoViewer;

        MotionDetector _detector;

        MediaConnector _connector;

        AVPlayer _videoPlayer;

        Color _color;

        int _motionCount;

        public MotionDetectorForm()
        {
            InitializeComponent();
        }

        void MotionDetectorForm_Load(object sender, EventArgs e)
        {
            _imageProvider = new DrawingImageProvider();
            _videoViewer = new VideoViewerWF();
            _connector = new MediaConnector();
            _detector = new MotionDetector();

            // subscribes to the event, which indicates when motions are being detected
            _detector.MotionDetection += _detector_MotionDetection;

            _color = Color.Red;

            // the videoplayer is being set, it will play the "video.mp4" file when being started
            _videoPlayer = new AVPlayer("video.mp4");

            SetVideoViewer();

            // connects the input video to the detector
            _connector.Connect(_videoPlayer.VideoChannel, _detector);

            // connects the detector to the output
            _connector.Connect(_detector, _imageProvider);

            _videoViewer.SetImageProvider(_imageProvider);

            // fills the ComboBox with the available colors
            cmb_DrawColor.DataSource = Enum.GetValues(typeof(MotionColor));
        }

        // occurrs, when motion is being detected or the detected motion ends
        void _detector_MotionDetection(object sender, MotionDetectionEvent e)
        {
            // stores if the event indicates the beginning or the ending of the motion
            if (e.Detection)
            {
                // increments the counter of detected motions
                _motionCount++;

                InvokeGuiThread(() => { lbl_MotionCount.Text = _motionCount.ToString(); });

                // indicates the motion within a simple panel
                panel_MotionIndicator.BackColor = _color;
            }
            else
                panel_MotionIndicator.BackColor = Color.Gray;
        }

        // sets the VideoViewerWF object programmatically
        void SetVideoViewer()
        {
            _videoViewer.Location = new Point(10, 10);
            _videoViewer.Size = new Size(375, 300);
            _videoViewer.BackColor = Color.Black;
            _videoViewer.TabStop = false;

            Controls.Add(_videoViewer);
        }

        // plays the video
        void btn_Play_Click(object sender, EventArgs e)
        {
            _videoViewer.Start();
            _videoPlayer.Start();
        }

        #region Detector

        void btn_Detect_Click(object sender, EventArgs e)
        {
            // _detector.IsRunning indicates if the MotionDetectorHandler is currently detecting or not
            if (!_detector.IsRunning)
            {
                // starts the detector
                _detector.Start();

                btn_Detect.Text = "Stop";
            }
            else
            {
                // stops the detector
                _detector.Stop();

                btn_Detect.Text = "Start";

                panel_MotionIndicator.BackColor = Color.Gray;
            }
        }

        // when the ComboBox of MotionColors is being set, it sets the detector's MotionColor property
        void cmb_DrawColor_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cmb_DrawColor.SelectedIndex)
            {
                case 0:
                    _detector.MotionColor = MotionColor.Red;
                    _color = Color.DarkRed;
                    break;
                case 1:
                    _detector.MotionColor = MotionColor.Green;
                    _color = Color.DarkGreen;
                    break;
                case 2:
                    _detector.MotionColor = MotionColor.Blue;
                    _color = Color.Blue;
                    break;
                default:
                    _detector.MotionColor = MotionColor.Red;
                    _color = Color.DarkRed;
                    break;
            }
        }

        void InvokeGuiThread(Action action)
        {
            Invoke(action);
        }

        void btn_MotionIntensity_Click(object sender, EventArgs e)
        {
            // how much difference should be found between two frames
            //on the same position (pixel) to be recognized as a possible part of a moving object.
            _detector.PixelIntensitySensitivy = Int32.Parse(tb_MotionIntensity.Text);
        }

        void btn_MotionAmount_Click(object sender, EventArgs e)
        {
            // how many pixeles should be detected as moving object parts to recognize motion
            _detector.PixelAmountSensitivy = Int32.Parse(tb_MotionAmount.Text);
        }

        void btn_MotionFrame_Click(object sender, EventArgs e)
        {
            // how many frames should be checked per seconds
            _detector.DetectionFPS = Int32.Parse(tb_MotionFrames.Text);
        }

        void radio_NoDraw_CheckedChanged(object sender, EventArgs e)
        {
            // sets the mode of highlighting
            _detector.HighlightMotion = HighlightMotion.None;
        }

        void radio_Draw_CheckedChanged(object sender, EventArgs e)
        {
            _detector.HighlightMotion = HighlightMotion.DrawAround;
        }

        void radio_HighLight_CheckedChanged(object sender, EventArgs e)
        {
            _detector.HighlightMotion = HighlightMotion.Highlight;
        }

        void btn_ResetCounter_Click(object sender, EventArgs e)
        {
            // resets the motion counter
            _motionCount = 0;
            InvokeGuiThread(() => { lbl_MotionCount.Text = _motionCount.ToString(); });
        }

        #endregion

    }
}
